import { Arch } from "electron-builder-util";
import { AppInfo } from "./appInfo";
import { Platform, Target, TargetSpecificOptions } from "./core";
import { AfterPackContext, Config, FileAssociation, PlatformSpecificBuildOptions } from "./metadata";
import { Packager } from "./packager";
import { PackagerOptions } from "./packagerApi";
import { AsyncTaskManager } from "./util/asyncTaskManager";
export declare abstract class PlatformPackager<DC extends PlatformSpecificBuildOptions> {
    readonly info: Packager;
    readonly packagerOptions: PackagerOptions;
    readonly projectDir: string;
    readonly buildResourcesDir: string;
    readonly config: Config;
    readonly platformSpecificBuildOptions: DC;
    readonly resourceList: Promise<Array<string>>;
    private readonly _resourceList;
    readonly abstract platform: Platform;
    readonly appInfo: AppInfo;
    constructor(info: Packager);
    readonly abstract defaultTarget: Array<string>;
    protected prepareAppInfo(appInfo: AppInfo): AppInfo;
    private static normalizePlatformSpecificBuildOptions(options);
    abstract createTargets(targets: Array<string>, mapper: (name: string, factory: (outDir: string) => Target) => void): void;
    protected getCscPassword(): string;
    protected doGetCscPassword(): string | undefined;
    readonly relativeBuildResourcesDirname: string;
    protected computeAppOutDir(outDir: string, arch: Arch): string;
    dispatchArtifactCreated(file: string, target: Target | null, arch: Arch | null, safeArtifactName?: string): void;
    pack(outDir: string, arch: Arch, targets: Array<Target>, taskManager: AsyncTaskManager): Promise<any>;
    protected packageInDistributableFormat(appOutDir: string, arch: Arch, targets: Array<Target>, taskManager: AsyncTaskManager): void;
    private getExtraFileMatchers(isResources, appOutDir, macroExpander, customBuildOptions);
    readonly electronDistMacOsAppName: string;
    readonly electronDistExecutableName: string;
    readonly electronDistMacOsExecutableName: string;
    protected doPack(outDir: string, appOutDir: string, platformName: string, arch: Arch, platformSpecificBuildOptions: DC, targets: Array<Target>): Promise<void>;
    private copyAppFiles(taskManager, asarOptions, resourcePath, outDir, platformSpecificBuildOptions, excludePatterns, macroExpander);
    protected postInitApp(packContext: AfterPackContext): Promise<any>;
    protected signApp(packContext: AfterPackContext): Promise<any>;
    getIconPath(): Promise<string | null>;
    private computeAsarOptions(customBuildOptions);
    getElectronSrcDir(dist: string): string;
    getElectronDestinationDir(appOutDir: string): string;
    getResourcesDir(appOutDir: string): string;
    getMacOsResourcesDir(appOutDir: string): string;
    private checkFileInPackage(resourcesDir, file, messagePrefix, isAsar);
    private sanityCheckPackage(appOutDir, isAsar);
    computeSafeArtifactName(ext: string, arch?: Arch | null, skipArchIfX64?: boolean): string;
    expandArtifactNamePattern(targetSpecificOptions: TargetSpecificOptions | null | undefined, ext: string, arch?: Arch | null, defaultPattern?: string, skipArchIfX64?: boolean): string;
    private computeArtifactName(pattern, ext, arch);
    expandMacro(pattern: string, arch?: string | null, extra?: any, isProductNameSanitized?: boolean): string;
    generateName(ext: string | null, arch: Arch, deployment: boolean, classifier?: string | null): string;
    generateName2(ext: string | null, classifier: string | null | undefined, deployment: boolean): string;
    getDefaultIcon(ext: string): Promise<string | null>;
    getTempFile(suffix: string): Promise<string>;
    getTempDir(suffix: string): Promise<string>;
    readonly fileAssociations: Array<FileAssociation>;
    getResource(custom: string | null | undefined, ...names: Array<string>): Promise<string | null>;
    readonly forceCodeSigning: boolean;
}
export declare function normalizeExt(ext: string): string;
